@@ -257,12 +257,19 @@ class APIConnection(HttpConnection):
         return self.send_request(action, body)
 
     def reset_instances(self, instances,
+                              login_mode=None,
+                              login_passwd=None,
+                              login_keypair=None,
                               **ignore):
         """ Reset one or monre instances to its initial state.
-            @param instances : An array including IDs of the instances you want to reset.
+            @param login_mode: login mode, only supported for linux instance, valid values are "keypair", "passwd".
+            @param login_passwd: if login_mode is "passwd", should be specified.
+            @param login_keypair: if login_mode is "keypair", should be specified.
+            @param instances : an array of instance ids you want to reset.
         """
         action = const.ACTION_RESET_INSTANCES
-        body = {'instances': instances}
+        valid_keys = ['instances', 'login_mode', 'login_passwd', 'login_keypair']
+        body = filter_out_none(locals(), valid_keys)
         if not self.req_checker.check_params(body,
                 required_params=['instances'],
                 integer_params=[],
@@ -760,20 +767,36 @@ class APIConnection(HttpConnection):
         return self.send_request(action, body)
 
     def modify_security_group_rule_attributes(self, security_group_rule,
-                                                    priority,
+                                                    priority=None,
                                                     security_group_rule_name=None,
+                                                    rule_action=None,
+                                                    direction=None,
+                                                    protocol=None,
+                                                    val1=None,
+                                                    val2=None,
+                                                    val3=None,
                                                     **ignore):
         """ Modify security group rule attributes.
             @param security_group_rule: the ID of the security group rule whose attributes you
                                       want to update.
             @param priority: priority [0 - 100].
             @param security_group_rule_name: name of the rule.
+
+            @param rule_action: "accept" or "drop".
+            @param direction: 0 for inbound; 1 for outbound.
+            @param protocol: supported protocols are "icmp", "tcp", "udp", "gre".
+            @param val1: for "icmp" protocol, this field is "icmp type";
+                         for "tcp/udp", it's "start port", empty means all.
+            @param val2: for "icmp" protocol, this field is "icmp code";
+                         for "tcp/udp", it's "end port", empty means all.
+            @param val3: ip network, e.g "1.2.3.0/24"
         """
         action = const.ACTION_MODIFY_SECURITY_GROUP_RULE_ATTRIBUTES
-        valid_keys = ['security_group_rule', 'priority', 'security_group_rule_name']
+        valid_keys = ['security_group_rule', 'priority', 'security_group_rule_name',
+                'rule_action', 'direction', 'protocol', 'val1', 'val2', 'val3']
         body = filter_out_none(locals(), valid_keys)
         if not self.req_checker.check_params(body,
-                required_params=['security_group_rule', 'priority'],
+                required_params=['security_group_rule'],
                 integer_params=['priority'],
                 list_params=[]
                 ):
@@ -1049,17 +1072,24 @@ class APIConnection(HttpConnection):
     def join_router(self, vxnet,
                           router,
                           ip_network,
+                          manager_ip=None,
+                          dyn_ip_start=None,
+                          dyn_ip_end=None,
                           features=1,
                           **ignore):
         """ Connect vxnet to router.
             @param vxnet: the ID of vxnet that will join the router.
             @param router: the ID of the router the vxnet will join.
+            @param ip_network: the ip network in CSI format.
+            @param manager_ip: can be provided if DHCP feature is enabled.
+            @param dyn_ip_start: starting IP that allocated from DHCP server.
+            @param dyn_ip_end: ending IP that allocated from DHCP server.
             @param features: the feature the vxnet will enable in the router.
                              1 - dhcp server.
-            @param ip_network: the ip network in CSI format.
         """
         action = const.ACTION_JOIN_ROUTER
-        valid_keys = ['vxnet', 'router', 'ip_network', 'features']
+        valid_keys = ['vxnet', 'router', 'ip_network', 'manager_ip',
+                'dyn_ip_start', 'dyn_ip_end', 'features']
         body = filter_out_none(locals(), valid_keys)
         if not self.req_checker.check_params(body,
                 required_params=['vxnet', 'router', 'ip_network'],
@@ -1095,6 +1125,9 @@ class APIConnection(HttpConnection):
                                        security_group=None,
                                        router_name=None,
                                        description=None,
+                                       features=None,
+                                       dyn_ip_start=None,
+                                       dyn_ip_end=None,
                                        **ignore):
         """ Modify router attributes.
             @param router: the ID of router you want to modify its attributes.
@@ -1103,10 +1136,13 @@ class APIConnection(HttpConnection):
             @param security_group: the ID of the security_group you want to apply to router.
             @param router_name: the name of the router.
             @param description: the description of the router.
+            @param features: the features of vxnet you want to re-define. 1: enable DHCP; 0: disable DHCP
+            @param dyn_ip_start: starting IP that allocated from DHCP server
+            @param dyn_ip_end: ending IP that allocated from DHCP server
         """
         action = const.ACTION_MODIFY_ROUTER_ATTRIBUTES
-        valid_keys = ['router', 'vxnet', 'eip', 'security_group',
-                'router_name', 'description']
+        valid_keys = ['router', 'vxnet', 'eip', 'security_group', 'features',
+                'router_name', 'description', 'dyn_ip_start', 'dyn_ip_end']
         body = filter_out_none(locals(), valid_keys)
         if not self.req_checker.check_params(body,
                 required_params=['router'],
